@@ -89,124 +89,6 @@ static int handle_error(prelude_error_code_t code, PGconn *conn)
 }
 
 
-static int sql_open(preludedb_sql_settings_t *settings, void **session)
-{
-        int ret;
-        PGconn *conn;
-
-        conn = PQsetdbLogin(preludedb_sql_settings_get_host(settings),
-                            preludedb_sql_settings_get_port(settings),
-                            NULL,
-                            NULL,
-                            preludedb_sql_settings_get_name(settings),
-                            preludedb_sql_settings_get_user(settings),
-                            preludedb_sql_settings_get_pass(settings));
-
-        if ( PQstatus(conn) == CONNECTION_BAD ) {
-                ret = handle_error(PRELUDEDB_ERROR_CONNECTION, conn);
-                PQfinish(conn);
-                return ret;
-        }
-
-        *session = conn;
-
-        return 0;
-}
-
-
-
-static void sql_close(void *session)
-{
-        PQfinish(session);
-}
-
-
-static int sql_escape(void *session, const char *input, size_t input_size, char **output)
-{
-        size_t rsize;
-
-        rsize = input_size * 2 + 3;
-        if ( rsize <= input_size )
-                return preludedb_error(PRELUDEDB_ERROR_GENERIC);
-        
-        *output = malloc(rsize);
-        if ( ! *output )
-                return preludedb_error_from_errno(errno);
-
-        (*output)[0] = '\'';
-
-        rsize = PQescapeString((*output) + 1, input, input_size);
-
-        (*output)[rsize + 1] = '\'';
-        (*output)[rsize + 2] = '\0';
-
-        return 0;
-}
-
-
-
-static int sql_escape_binary(void *session, const unsigned char *input, size_t input_size, char **output)
-{
-        prelude_string_t *string;
-        size_t rsize, dummy;
-        unsigned char *ptr;
-        int ret;
-
-        rsize = input_size * 2 + 3;
-        if ( rsize <= input_size )
-                return preludedb_error(PRELUDEDB_ERROR_GENERIC);
-
-        ret = prelude_string_new(&string);
-        if ( ret < 0 )
-                return ret;
-        
-        ptr = PQescapeBytea(input, input_size, &dummy);
-        if ( ! ptr ) {
-                prelude_string_destroy(string);
-                return -1;
-        }
-        
-        ret = prelude_string_sprintf(string, "'%s'", ptr);
-        free(ptr);
-
-        if ( ret < 0 ) {
-                prelude_string_destroy(string);
-                return ret;
-        }
-
-        ret = prelude_string_get_string_released(string, output);
-
-        prelude_string_destroy(string);
-
-        return 0;
-}
-
-
-
-static int sql_unescape_binary(void *session, const char *input, unsigned char **output, size_t *output_size)
-{
-        *output = PQunescapeBytea((const unsigned char *) input, output_size);
-        if ( ! *output )
-                return preludedb_error_from_errno(errno);
-
-        return 0;
-}
-
-
-
-static int sql_build_limit_offset_string(void *session, int limit, int offset, prelude_string_t *output)
-{
-        if ( limit >= 0 ) {
-                if ( offset >= 0 )
-                        return prelude_string_sprintf(output, " LIMIT %d OFFSET %d", limit, offset);
-                
-                return prelude_string_sprintf(output, " LIMIT %d", limit);
-        }
-
-        return 0;
-}
-
-
 
 static int sql_query(void *session, const char *query, void **resource)
 {
@@ -240,6 +122,118 @@ static int sql_query(void *session, const char *query, void **resource)
 }
 
 
+static int sql_open(preludedb_sql_settings_t *settings, void **session)
+{
+        int ret;
+        PGconn *conn;
+
+        conn = PQsetdbLogin(preludedb_sql_settings_get_host(settings),
+                            preludedb_sql_settings_get_port(settings),
+                            NULL,
+                            NULL,
+                            preludedb_sql_settings_get_name(settings),
+                            preludedb_sql_settings_get_user(settings),
+                            preludedb_sql_settings_get_pass(settings));
+
+        if ( PQstatus(conn) == CONNECTION_BAD ) {
+                ret = handle_error(PRELUDEDB_ERROR_CONNECTION, conn);
+                PQfinish(conn);
+                return ret;
+        }
+
+        *session = conn;
+
+        return sql_query(conn, "SET DATESTYLE TO 'ISO'", NULL);
+}
+
+
+
+static void sql_close(void *session)
+{
+        PQfinish(session);
+}
+
+
+static int sql_escape(void *session, const char *input, size_t input_size, char **output)
+{
+        size_t rsize;
+
+        rsize = input_size * 2 + 3;
+        if ( rsize <= input_size )
+                return preludedb_error(PRELUDEDB_ERROR_GENERIC);
+        
+        *output = malloc(rsize);
+        if ( ! *output )
+                return preludedb_error_from_errno(errno);
+
+        (*output)[0] = '\'';
+
+        rsize = PQescapeString((*output) + 1, input, input_size);
+
+        (*output)[rsize + 1] = '\'';
+        (*output)[rsize + 2] = '\0';
+        
+        return 0;
+}
+
+
+static int sql_escape_binary(void *session, const unsigned char *input, size_t input_size, char **output)
+{
+        int ret;
+        size_t dummy;
+        unsigned char *ptr;
+        prelude_string_t *string;
+
+        ret = prelude_string_new(&string);
+        if ( ret < 0 )
+                return ret;
+        
+        ptr = PQescapeBytea(input, input_size, &dummy);
+        if ( ! ptr ) {
+                prelude_string_destroy(string);
+                return -1;
+        }
+        
+        ret = prelude_string_sprintf(string, "'%s'", ptr);
+        free(ptr);
+
+        if ( ret < 0 ) {
+                prelude_string_destroy(string);
+                return ret;
+        }
+
+        ret = prelude_string_get_string_released(string, output);
+        prelude_string_destroy(string);
+        
+        return 0;
+}
+
+
+
+static int sql_unescape_binary(void *session, const char *input, unsigned char **output, size_t *output_size)
+{
+        *output = PQunescapeBytea((const unsigned char *) input, output_size);
+        if ( ! *output )
+                return preludedb_error_from_errno(errno);
+
+        return 0;
+}
+
+
+
+static int sql_build_limit_offset_string(void *session, int limit, int offset, prelude_string_t *output)
+{
+        if ( limit >= 0 ) {
+                if ( offset >= 0 )
+                        return prelude_string_sprintf(output, " LIMIT %d OFFSET %d", limit, offset);
+                
+                return prelude_string_sprintf(output, " LIMIT %d", limit);
+        }
+
+        return 0;
+}
+
+
 
 static void sql_resource_destroy(void *session, void *resource)
 {
